#include "HelloAPI/HelloAPI.hpp"
#include "HelloLib/HelloLib.hpp"

#include <iostream>
#include <stdexcept>

#if defined( TARGET_PLATFORM_WINDOWS )
#	include <windows.h>
#elif defined( TARGET_PLATFORM_TYPE_POSIX ) && !defined( PROVIDES_TESTAPI )
#	include <dlfcn.h>
#endif

#if defined( _WIN32 ) && !defined( _CONSOLE )
int        main( int argc, char * argv[] );
int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR lpCmdLine, INT nCmdShow ) { return main( 0, nullptr ); }
#endif

int main( int argc, char * argv[] )
{
#if defined( HARDLINK_HELLOAPI )
	std::cout << "STATIC LINK\n";
	hello_from_plugin();
#else
	std::cout << "DYNAMIC LINK\n";
	hello_import_t hello_from_plugin = nullptr;

#	ifdef TARGET_PLATFORM_WINDOWS
	{
		HMODULE plugin = LoadLibraryW( L"HelloPlugin.dll" );
		if( !plugin ) throw std::runtime_error( "Failed To Load 'HelloPlugin' Module" );
		hello_from_plugin = ( hello_import_t )( GetProcAddress( plugin, "hello_from_plugin" ) );
	}
#	elif defined( TARGET_PLATFORM_TYPE_POSIX )
	{
		void * plugin = dlopen( "HelloPlugin.so", RTLD_LAZY );
		if( !plugin ) throw std::runtime_error( "Failed To Load 'HelloPlugin' Module" );
		hello_from_plugin = ( hello_import_t )( dlsym( plugin, "hello_from_plugin" ) );
	}
#	endif

	if( !hello_from_plugin ) throw std::runtime_error( "Failed To Load 'hello_from_plugin' Symbol" );
	hello_from_plugin();
#endif

	hello_from_library();
	return 0;
}
